/* libzzwin
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2000-2006, Toni Thomsson <toni@tonjac.org> 
*/

#ifndef zz_nt_service_h
#define zz_nt_service_h

#include <winsock2.h>
#include <zz_string.h>
#include <zz_log.h>
#include <zz_win_exception.h>
#include <windows.h>

/**
	Windows service

	@author Toni Thomsson, toni@tonjac.org
*/

class ZZ_API CzzNTService
{
public:

	/* 
		Description:
		Skapar ett NT-tjnst objekt

		Remarks:
		Anvnd ej direkt: subklassa !

		Throws:
		CzzWinException

		Example:
		#include "myservice.h"

		void _CRTAPI1 main( int argc, char** argv )
		{
			 try
			 {
				CMyService my( "myservice", "myservice", "My service", "" );

				// Switch args...
				...

				// Install service
				my.Install();
				return;

				...

				// Remove service
				my.Remove();
				return;

				...

				// Debug
				my.Debug( argc, argv );
				return;

				...

				// Start service
				my.Start();

			}
			catch( CzzException& e )
			{
				...
			}

		}	
	*/
	CzzNTService( const char* appname,		// IN, applikationsnamn
					const char* svc,		// IN, tjnstnamn
					const char* args,		// IN, argument
					const char* display,	// IN, beskrivnings text
					const char* dep,		// IN, beroendestrng, se SDK
					const char* descr,		// IN, beskrivningsstrng
					DWORD starttype );

	CzzNTService( const char* appname,		// IN, applikationsnamn
					const char* svc,		// IN, tjnstnamn
					const char* args,		// IN, argument
					const char* display,	// IN, beskrivnings text
					const char* dep,		// IN, beroendestrng, se SDK
					const char* descr		// IN, beskrivningsstrng
					);

	CzzNTService( const char* appname,		// IN, applikationsnamn
					const char* svc,		// IN, tjnstnamn
					const char* args,		// IN, argument
					const char* display,	// IN, beskrivnings text
					const char* dep  		// IN, beroendestrng, se SDK
					);

	virtual ~CzzNTService();

	/*
		Description:
		Installerar tjnsten i windows registry

		Remarks:
		verlagringsbar

		Throws:
		CzzWinException
	*/
	virtual void Install( void );

	/*
		Description:
		Tar bort tjnsten frn windows registry

		Remarks:
		verlagringsbar

		Throws:
		CzzWinException
	*/
	virtual void Remove( void );

	/*
		Description:
		Startar i "debug-lge" vanlig konsol

		Remarks:
		verlagringsbar

		Throws:
		CzzWinException
	*/
	virtual void Debug( int argc,		// IN, antal argument
						char** argv		// IN, argumentlista
						);

	/*
		Description:
		Startar tjnsten

		Remarks:
		verlagringsbar

		Throws:
		CzzWinException
	*/
	virtual void Start( void );

	// Description: Tjnstnamn
	CzzString& GetServiceName( void ) { return m_ServiceName; };
	
	// Description: Applikationsnamn
	CzzString& GetAppName( void ) { return m_AppName; };

	// Description: Tjnstestatus, se SDK
	SERVICE_STATUS& GetStatus( void ) { return m_Status; };

	// Description: Logg
	CzzLog& GetLog( void ) { return m_Log; };
		
	/*
		Description:
		Anropas nr tjnsten startas

		Remarks:
		verlagringsbar
	*/
	virtual void OnStart( int argc,			// IN, antal argument
							char** argv		// IN, argumentlista
							);

	/*
		Description:
		Anropas nr tjnsten stoppas

		Remarks:
		verlagringsbar
	*/
	virtual void OnStop( void );

	// Description: Rapporterar status till SCM
	void ReportStatus( DWORD state, DWORD exitcode, DWORD waithint );

	// Description: Registrerar tjnsten
	void Register( void );

protected:
	
	void Init( void );
	void Exit( void );

	SERVICE_STATUS m_Status;
	SERVICE_STATUS_HANDLE m_Handle;
	CzzLog m_Log;
	CzzString m_AppName;
	CzzString m_ServiceName;
	CzzString m_Args;
	CzzString m_DisplayName;
	CzzString m_Description;
	CzzString m_Dependencies;
	bool m_Debug;
	DWORD m_Error;
	DWORD m_CheckPoint;
	DWORD m_StartType;	
	HANDLE m_StopEvent;

private:
	
	static CzzNTService* m_pThis;

	static BOOL WINAPI ConsoleHandler( DWORD type );
	static VOID WINAPI ControlHandler( DWORD code );
	static void WINAPI Entrypoint( DWORD argc, LPSTR* argv );
};

#endif // zz_nt_service_h

